
/*
 * $Id$
 *
 * DEBUG: section ?    Storetree functions
 * AUTHOR: Yury Shevchuk
 *
 * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
 * ----------------------------------------------------------
 *
 *  Squid is the result of efforts by numerous individuals from the
 *  Internet community.  Development is led by Duane Wessels of the
 *  National Laboratory for Applied Network Research and funded by the
 *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
 *  the Regents of the University of California.  Please see the
 *  COPYRIGHT file for full details.  Squid incorporates software
 *  developed and/or copyrighted by other sources.  Please see the
 *  CREDITS file for full details.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 */

#include "squid.h"


/* Remove StoreEntry from memory while keeping the disk object.  Save
   important e->lastref and e->expires info in the file's stat data,
   so expiration process (storeTreeDirClean) can make decisions based
   on them without reading the file.

   For reiserfs_raw, we avoid setting attributes now.  They has been
   set on object creation, and the only change we may need now is
   update lastref attribute which is currently set to the time of file
   creation or last open.  (Hm, we must update lastref for hot objects
   periodically somewhere, or they might get GCed on the disk.)  */

static void
storeTreeRelease(StoreEntry *e)
{
    assert (e->lock_count == 0);

    if (EBIT_TEST (e->flags, ENTRY_TENTATIVE)) {
	/* If we are asked to release tentative entry, this means it
	   wasn't succesful (no file found for this entry).  It is no
	   longer tentative, it is simply dummy StoreEntry. */
	e->swap_status = SWAPOUT_NONE;
	e->swap_filen = -1;
	EBIT_CLR (e->flags, ENTRY_TENTATIVE);
	return;
    }

    if ((e->flags &~ (1L<<ENTRY_VALIDATED))
	!= ((1L<<ENTRY_CACHABLE)|(1L<<ENTRY_DISPATCHED))) {
	/* Uncommon, leave in memory */
	debug(86, 3) ("storeTreeRelease: leaving in memory entry with uncommon flags %s\n",
		      storeEntryFlags(e));
	return;
    }

    storeSetMemStatus(e, NOT_IN_MEMORY);
    destroy_StoreEntry(e);
}
